local M = {}

local ct    = nil
local c     = nil
local paint = {}

local bestLap = 0
local lastLap = 0
local lastSector = 0
local sectorTimes = {}

local function initPaint()
    local fontName = "Roboto-Light.ttf"
    local tf = SkTypeface.CreateFromFile(fontName, kNormal)
    
    paint.fontNormal = SkPaint()
    paint.fontNormal:setAntiAlias(true)
    paint.fontNormal:setARGB(255,0,0,150)
	paint.fontNormal:setTextAlign(kLeft_Align)
    paint.fontNormal:setTextSize(20)
    paint.fontNormal:setTypeface(tf)
	
	paint.fontSmall = SkPaint()
    paint.fontSmall:setAntiAlias(true)
    paint.fontSmall:setARGB(255,0,0,150)
	paint.fontSmall:setTextAlign(kLeft_Align)
    paint.fontSmall:setTextSize(16)
    paint.fontSmall:setTypeface(tf)
end

local function printTimes()
    if not ct then
        ct = BeamEngine.canvasTexture
    end
    c = ct:getCanvas()
    ct:clear()
    
    if ( paint.fontNormal == nil ) then
        initPaint()
    end
	
	bestLapTxt = string.format("Best: %.3f", bestLap)
	lastLapTxt = string.format("Last: %.3f", lastLap)
	lastSectorText = string.format("Sector: %.3f", lastSector)
	
    c:drawText(bestLapTxt, string.len(bestLapTxt), 20, 30, paint.fontNormal)
	c:drawText(lastLapTxt, string.len(lastLapTxt), 20, 52, paint.fontNormal)
	c:drawText(lastSectorText, string.len(lastSectorText), 20, 72, paint.fontSmall)
end

local function lapComplete(total)
	for i=1, total-1 do
		if sectorTimes[i] == nil or sectorTimes[i+1] == nil then
			return false
		elseif sectorTimes[i] > sectorTimes[i+1] then
			return false
		end
	end
	
	return true
end

local function loadTimes()
	dofile( "lua/laptimer/table.save-1.0.lua" )
	local bestLaps, err = table.load( "lua/laptimer/laptimes.data" )
	if err == nil then
		return bestLaps
	else
		print("Error loading map: "..err)
		return {}
	end
end

local function loadTime(levelname)
	local bestLaps = loadTimes()
	if bestLaps[levelname] ~= nil then
		bestLap = bestLaps[levelname]
	else
		bestLap = 0
	end
end

local function saveTime(levelname, time)
	dofile( "lua/laptimer/table.save-1.0.lua" )
	bestLaps = loadTimes()
	bestLaps[levelname] = time
	table.save( bestLaps, "lua/laptimer/laptimes.data" )
end

local function update(sector, total, levelname)
    local currentTime = os.clock()
	if lapComplete(total) then
		lastLap = currentTime-sectorTimes[1]
		lastSector = 0
		if bestLap == 0 or lastLap < bestLap then
			bestLap = lastLap
			
			if levelname ~= nil then
				saveTime(levelname, bestLap)
			end
		end
	end
	
	sectorTimes[sector] = currentTime
	if sectorTimes[1] ~= nil then
		lastSector = sectorTimes[sector]-sectorTimes[1]
	end
end

M.update     = update
M.printTimes  = printTimes
M.loadTime = loadTime

return M